@@ -52,8 +52,8 @@ class Inventory(Workflow, ModelSQL, ModelView):
         super(Inventory, cls).__setup__()
         cls._order.insert(0, ('date', 'DESC'))
         cls._error_messages.update({
-                'delete_cancel': 'Inventory "%s" must be cancelled before ' \
-                    'deletion!',
+                'delete_cancel': ('Inventory "%s" must be cancelled before '
+                    'deletion.'),
                 })
         cls._transitions |= set((
                 ('draft', 'done'),
@@ -113,9 +113,15 @@ class Inventory(Workflow, ModelSQL, ModelView):
     @ModelView.button
     @Workflow.transition('done')
     def confirm(self, inventories):
+        Move = Pool().get('stock.move')
+        move_ids = []
         for inventory in inventories:
             for line in inventory.lines:
-                line.create_move()
+                move_id = line.create_move()
+                if move_id:
+                    move_ids.append(move_id)
+        if move_ids:
+            Move.do(Move.browse(move_ids))
 
     @classmethod
     @ModelView.button
@@ -149,8 +155,8 @@ class Inventory(Workflow, ModelSQL, ModelView):
             new_inventories.append(new_inventory)
         return new_inventories
 
-    @classmethod
-    def complete_lines(self, inventories):
+    @staticmethod
+    def complete_lines(inventories):
         '''
         Complete or update the inventories
         '''
@@ -194,14 +200,17 @@ class Inventory(Workflow, ModelSQL, ModelView):
                     Line.write([line], values)
 
             # Create lines if needed
+            to_create = []
             for product_id in product_qty:
                 if (product2type[product_id] != 'goods'
-                        and not product2consumable[product_id]):
+                        or product2consumable[product_id]):
                     continue
                 quantity, uom_id = product_qty[product_id]
                 values = Line.create_values4complete(product_id, inventory,
                     quantity, uom_id)
-                Line.create(values)
+                to_create.append(values)
+            if to_create:
+                Line.create(to_create)
 
 
 class InventoryLine(ModelSQL, ModelView):
@@ -231,9 +240,9 @@ class InventoryLine(ModelSQL, ModelView):
         super(InventoryLine, cls).__setup__()
         cls._sql_constraints += [
             ('check_line_qty_pos',
-                'CHECK(quantity >= 0.0)', 'Line quantity must be positive!'),
+                'CHECK(quantity >= 0.0)', 'Line quantity must be positive.'),
             ('inventory_product_uniq', 'UNIQUE(inventory, product)',
-                'Product must be unique by inventory!'),
+                'Product must be unique by inventory.'),
             ]
         cls._order.insert(0, ('product', 'ASC'))
 
@@ -263,9 +272,7 @@ class InventoryLine(ModelSQL, ModelView):
     @classmethod
     def cancel_move(cls, lines):
         Move = Pool().get('stock.move')
-        Move.write([l.move for l in lines if l.move], {
-            'state': 'cancel',
-            })
+        Move.cancel([l.move for l in lines if l.move])
         Move.delete([l.move for l in lines if l.move])
         cls.write([l for l in lines if l.move], {
             'move': None,
@@ -290,16 +297,15 @@ class InventoryLine(ModelSQL, ModelView):
             (from_location, to_location, delta_qty) = \
                 (to_location, from_location, -delta_qty)
 
-        move = Move.create({
-            'from_location': from_location,
-            'to_location': to_location,
-            'quantity': delta_qty,
-            'product': self.product.id,
-            'uom': self.uom.id,
-            'company': self.inventory.company.id,
-            'state': 'done',
-            'effective_date': self.inventory.date,
-            })
+        move, = Move.create([{
+                    'from_location': from_location,
+                    'to_location': to_location,
+                    'quantity': delta_qty,
+                    'product': self.product.id,
+                    'uom': self.uom.id,
+                    'company': self.inventory.company.id,
+                    'effective_date': self.inventory.date,
+                    }])
         self.move = move
         self.save()
         return move.id
