@@ -1,8 +1,8 @@
 #This file is part of Tryton.  The COPYRIGHT file at the top level of
 #this repository contains the full copyright notices and license terms.
 from decimal import Decimal
-from functools import reduce
-from trytond.model import ModelView, ModelSQL, fields
+from functools import reduce, partial
+from trytond.model import Workflow, ModelView, ModelSQL, fields
 from trytond.backend import TableHandler
 from trytond.pyson import In, Eval, Not, Equal, If, Get, Bool
 from trytond.transaction import Transaction
@@ -16,7 +16,7 @@ STATES = {
 DEPENDS = ['state']
 
 
-class Move(ModelSQL, ModelView):
+class Move(Workflow, ModelSQL, ModelView):
     "Stock Move"
     __name__ = 'stock.move'
     _order_name = 'product'
@@ -50,30 +50,17 @@ class Move(ModelSQL, ModelView):
     to_location = fields.Many2One("stock.location", "To Location", select=True,
         required=True, states=STATES, depends=DEPENDS,
         domain=[('type', 'not in', ('warehouse', 'view'))])
-    shipment_in = fields.Many2One('stock.shipment.in', 'Supplier Shipment',
-        domain=[('company', '=', Eval('company'))], depends=['company'],
-        readonly=True, select=True, ondelete='CASCADE')
-    shipment_out = fields.Many2One('stock.shipment.out', 'Customer Shipment',
-        domain=[('company', '=', Eval('company'))], depends=['company'],
-        readonly=True, select=True, ondelete='CASCADE')
-    shipment_out_return = fields.Many2One('stock.shipment.out.return',
-        'Customer Return Shipment', readonly=True, select=True,
-        domain=[('company', '=', Eval('company'))], depends=['company'],
-        ondelete='CASCADE')
-    shipment_in_return = fields.Many2One('stock.shipment.in.return',
-        'Supplier Return Shipment', readonly=True, select=True,
-        domain=[('company', '=', Eval('company'))], depends=['company'],
-        ondelete='CASCADE')
-    shipment_internal = fields.Many2One('stock.shipment.internal',
-        'Internal Shipment', readonly=True, select=True, ondelete='CASCADE',
-        domain=[('company', '=', Eval('company'))], depends=['company'])
+    shipment = fields.Reference('Shipment', selection='get_shipment',
+        readonly=True, select=True)
+    origin = fields.Reference('Origin', selection='get_origin', select=True,
+        states={
+            'readonly': Eval('state') != 'draft',
+            },
+        depends=['state'])
     planned_date = fields.Date("Planned Date", states={
             'readonly': (In(Eval('state'), ['cancel', 'assigned', 'done'])
-                | Eval('shipment_in') | Eval('shipment_out')
-                | Eval('shipment_in_return') | Eval('shipment_out_return')
-                | Eval('shipment_internal'))
-            }, depends=['state', 'shipment_in', 'shipment_out',
-            'shipment_in_return', 'shipment_out_return', 'shipment_internal'],
+                | Eval('shipment'))
+            }, depends=['state', 'shipment'],
         select=True)
     effective_date = fields.Date("Effective Date", readonly=True, select=True)
     state = fields.Selection([
@@ -122,33 +109,43 @@ class Move(ModelSQL, ModelView):
             ('check_from_to_locations',
                 'CHECK(from_location != to_location)',
                 'Source and destination location must be different'),
-            ('check_shipment',
-                'CHECK((COALESCE(shipment_in, 0) / COALESCE(shipment_in, 1) ' \
-                        '+ COALESCE(shipment_out, 0) / ' \
-                            'COALESCE(shipment_out, 1) ' \
-                        '+ COALESCE(shipment_internal, 0) / ' \
-                            'COALESCE(shipment_internal, 1) ' \
-                        '+ COALESCE(shipment_in_return, 0) / ' \
-                            'COALESCE(shipment_in_return, 1) ' \
-                        '+ COALESCE(shipment_out_return, 0) / ' \
-                            'COALESCE(shipment_out_return, 1)) ' \
-                        '<= 1)',
-                'Move can be on only one Shipment'),
-        ]
-        cls._constraints += [
-            ('check_period_closed', 'period_closed'),
-        ]
+            ]
         cls._order[0] = ('id', 'DESC')
         cls._error_messages.update({
-            'set_state_draft': 'You can not set state to draft!',
-            'set_state_assigned': 'You can not set state to assigned!',
-            'set_state_done': 'You can not set state to done!',
-            'del_draft_cancel': 'You can only delete draft ' \
-                'or cancelled moves!',
-            'period_closed': 'You can not modify move in closed period!',
-            'modify_assigned_done_cancel': ('You can not modify a move '
-                'in the state: "Assigned", "Done" or "Cancel"'),
+            'set_state_draft': ('You can not set stock move "%s" to draft '
+                'state.'),
+            'set_state_assigned': ('You can not set stock move "%s" to '
+                'assigned state.'),
+            'set_state_done': 'You can not set stock move "%s" to done state.',
+            'del_draft_cancel': ('You can not delete stock move "%s" because '
+                'it is not in draft or cancelled state.'),
+            'period_closed': ('You can not modify move "%(move)s" because '
+                'period "%(period)s" is closed.'),
+            'modify_assigned_done_cancel': ('You can not modify stock move "%s"'
+                'because it is in "Assigned", "Done" or "Cancel" state.'),
             })
+        cls._transitions |= set((
+                ('draft', 'assigned'),
+                ('draft', 'done'),
+                ('draft', 'cancel'),
+                ('assigned', 'draft'),
+                ('assigned', 'done'),
+                ('assigned', 'cancel'),
+                ))
+        cls._buttons.update({
+                'cancel': {
+                    'invisible': ~Eval('state').in_(['draft', 'assigned']),
+                    },
+                'draft': {
+                    'invisible': ~Eval('state').in_(['assigned']),
+                    },
+                'assign': {
+                    'invisible': ~Eval('state').in_(['assigned']),
+                    },
+                'do': {
+                    'invisible': ~Eval('state').in_(['draft', 'assigned']),
+                    },
+                })
 
     @classmethod
     def __register__(cls, module_name):
@@ -180,9 +177,10 @@ class Move(ModelSQL, ModelView):
                 for move in moves:
                     internal_quantity = cls._get_internal_quantity(
                             move.quantity, move.uom, move.product)
-                    cls.write([move], {
-                        'internal_quantity': internal_quantity,
-                        })
+                    cursor.execute(
+                        'UPDATE "' + cls._table + '" '
+                        'SET internal_quantity = %s '
+                        'WHERE id = %s', (internal_quantity, move.id))
             table = TableHandler(cursor, cls, module_name)
             table.not_null_action('internal_quantity', action='add')
 
@@ -190,6 +188,22 @@ class Move(ModelSQL, ModelView):
         table = TableHandler(cursor, cls, module_name)
         table.drop_constraint('check_packing_in_out')
 
+        # Migration from 2.6: merge all shipments
+        table.drop_constraint('check_shipment')
+        shipments = {
+            'shipment_in': 'stock.shipment.in',
+            'shipment_out': 'stock.shipment.out',
+            'shipment_out_return': 'stock.shipment.out.return',
+            'shipment_in_return': 'stock.shipment.in.return',
+            'shipment_internal': 'stock.shipment.internal',
+            }
+        for column, model in shipments.iteritems():
+            if table.column_exist(column):
+                cursor.execute('UPDATE "' + cls._table + '" '
+                    'SET shipment = \'' + model + ',\' || "' + column + '" '
+                    'WHERE "' + column + '" IS NOT NULL')
+                table.drop_column(column)
+
         # Add index on create_date
         table.index_action('create_date', action='add')
 
@@ -281,8 +295,51 @@ class Move(ModelSQL, ModelView):
         if (self.to_location
                 and self.to_location.type == 'customer'):
             return True
+        if (self.from_location and self.to_location
+                and self.from_location.type == 'storage'
+                and self.to_location.type == 'supplier'):
+            return True
         return False
 
+    @staticmethod
+    def _get_shipment():
+        'Return list of Model names for shipment Reference'
+        return [
+            'stock.shipment.in',
+            'stock.shipment.out',
+            'stock.shipment.out.return',
+            'stock.shipment.in.return',
+            'stock.shipment.internal',
+            ]
+
+    @classmethod
+    def get_shipment(cls):
+        Model = Pool().get('ir.model')
+        models = cls._get_shipment()
+        models = Model.search([
+                ('model', 'in', models),
+                ])
+        return [(None, '')] + [(m.model, m.name) for m in models]
+
+    @staticmethod
+    def _get_origin():
+        'Return list of Model names for origin Reference'
+        return []
+
+    @classmethod
+    def get_origin(cls):
+        Model = Pool().get('ir.model')
+        models = cls._get_origin()
+        models = Model.search([
+                ('model', 'in', models),
+                ])
+        return [(None, '')] + [(m.model, m.name) for m in models]
+
+    @classmethod
+    def validate(cls, moves):
+        super(Move, cls).validate(moves)
+        cls.check_period_closed(moves)
+
     @classmethod
     def check_period_closed(cls, moves):
         Period = Pool().get('stock.period')
@@ -295,8 +352,10 @@ class Move(ModelSQL, ModelView):
                 date = (move.effective_date if move.effective_date
                     else move.planned_date)
                 if date and date < period.date:
-                    return False
-        return True
+                    cls.raise_user_error('period_closed', {
+                            'move': move.rec_name,
+                            'period': period.rec_name,
+                            })
 
     def get_rec_name(self, name):
         return ("%s%s %s"
@@ -306,13 +365,10 @@ class Move(ModelSQL, ModelView):
     def search_rec_name(cls, name, clause):
         return [('product',) + clause[1:]]
 
-    @classmethod
-    def _update_product_cost_price(cls, product_id, quantity, uom, unit_price,
-            currency, company, date):
+    def _update_product_cost_price(self, direction):
         """
         Update the cost price on the given product.
-        The quantity must be positive if incoming and negative if outgoing.
-        The date is for the currency rate calculation.
+        The direction must be "in" if incoming and "out" if outgoing.
         """
         pool = Pool()
         Uom = pool.get('product.uom')
@@ -320,24 +376,21 @@ class Move(ModelSQL, ModelView):
         ProductTemplate = pool.get('product.template')
         Location = pool.get('stock.location')
         Currency = pool.get('currency.currency')
-        Company = pool.get('company.company')
         Date = pool.get('ir.date')
 
-        if isinstance(uom, (int, long)):
-            uom = Uom(uom)
-        if isinstance(currency, (int, long)):
-            currency = Currency(currency)
-        if isinstance(company, (int, long)):
-            company = Company(company)
-
+        if direction == 'in':
+            quantity = self.quantity
+        elif direction == 'out':
+            quantity = -self.quantity
         context = {}
-        context['locations'] = Location.search([
+        locations = Location.search([
                 ('type', '=', 'storage'),
                 ])
+        context['locations'] = [l.id for l in locations]
         context['stock_date_end'] = Date.today()
         with Transaction().set_context(context):
-            product = Product(product_id)
-        qty = Uom.compute_qty(uom, quantity, product.default_uom)
+            product = Product(self.product.id)
+        qty = Uom.compute_qty(self.uom, quantity, product.default_uom)
 
         qty = Decimal(str(qty))
         if hasattr(Product, 'cost_price'):
@@ -346,11 +399,12 @@ class Move(ModelSQL, ModelView):
             product_qty = product.template.quantity
         product_qty = Decimal(str(product_qty))
         # convert wrt currency
-        with Transaction().set_context(date=date):
-            unit_price = Currency.compute(currency, unit_price,
-                company.currency, round=False)
+        with Transaction().set_context(date=self.effective_date):
+            unit_price = Currency.compute(self.currency, self.unit_price,
+                self.company.currency, round=False)
         # convert wrt to the uom
-        unit_price = Uom.compute_price(uom, unit_price, product.default_uom)
+        unit_price = Uom.compute_price(self.uom, unit_price,
+            product.default_uom)
         if product_qty + qty != Decimal('0.0'):
             new_cost_price = (
                 (product.cost_price * product_qty) + (unit_price * qty)
@@ -360,13 +414,15 @@ class Move(ModelSQL, ModelView):
 
         if hasattr(Product, 'cost_price'):
             digits = Product.cost_price.digits
+            write = partial(Product.write, [product])
         else:
             digits = ProductTemplate.cost_price.digits
+            write = partial(ProductTemplate.write, [product.template])
         new_cost_price = new_cost_price.quantize(
             Decimal(str(10.0 ** -digits[1])))
 
         with Transaction().set_user(0, set_context=True):
-            Product.write([product], {
+            write({
                     'cost_price': new_cost_price,
                     })
 
@@ -378,138 +434,116 @@ class Move(ModelSQL, ModelView):
         return internal_quantity
 
     @classmethod
-    def create(cls, vals):
+    @ModelView.button
+    @Workflow.transition('draft')
+    def draft(cls, moves):
+        pass
+
+    @classmethod
+    @ModelView.button
+    @Workflow.transition('assigned')
+    def assign(cls, moves):
         pool = Pool()
-        Location = pool.get('stock.location')
-        Product = pool.get('product.product')
-        Uom = pool.get('product.uom')
         Date = pool.get('ir.date')
 
         today = Date.today()
-        vals = vals.copy()
-        effective_date = vals.get('effective_date') or today
+        for move in moves:
+            if not move.effective_date:
+                move.effective_date = today
+            move.save()
 
-        product = Product(vals['product'])
-        if vals.get('state') == 'done':
-            vals['effective_date'] = effective_date
-            currency_id = vals.get('currency', cls.default_currency())
-            company_id = vals.get('company', cls.default_company())
-            from_location = Location(vals['from_location'])
-            to_location = Location(vals['to_location'])
-            if (from_location.type in ('supplier', 'production')
-                    and to_location.type == 'storage'
-                    and product.cost_price_method == 'average'):
-                cls._update_product_cost_price(vals['product'],
-                        vals['quantity'], vals['uom'], vals['unit_price'],
-                        currency_id, company_id, effective_date)
-            if (to_location.type == 'supplier'
-                    and from_location.type == 'storage'
-                    and product.cost_price_method == 'average'):
-                cls._update_product_cost_price(vals['product'],
-                        -vals['quantity'], vals['uom'], vals['unit_price'],
-                        currency_id, company_id, effective_date)
-            if not vals.get('cost_price'):
-                # Re-read product to get the updated cost_price
-                product = Product(vals['product'])
-                vals['cost_price'] = product.cost_price
+    @classmethod
+    @ModelView.button
+    @Workflow.transition('done')
+    def do(cls, moves):
+        pool = Pool()
+        Date = pool.get('ir.date')
 
-        elif vals.get('state') == 'assigned':
-            vals['effective_date'] = effective_date
+        today = Date.today()
+        for move in moves:
+            if not move.effective_date:
+                move.effective_date = today
+            if (move.from_location.type in ('supplier', 'production')
+                    and move.to_location.type == 'storage'
+                    and move.product.cost_price_method == 'average'):
+                move._update_product_cost_price('in')
+            elif (move.to_location.type == 'supplier'
+                    and move.from_location.type == 'storage'
+                    and move.product.cost_price_method == 'average'):
+                move._update_product_cost_price('out')
+            if not move.cost_price:
+                move.cost_price = move.product.cost_price
+            move.save()
 
-        uom = Uom(vals['uom'])
-        internal_quantity = cls._get_internal_quantity(vals['quantity'],
-            uom, product)
-        vals['internal_quantity'] = internal_quantity
-        return super(Move, cls).create(vals)
+    @classmethod
+    @ModelView.button
+    @Workflow.transition('cancel')
+    def cancel(cls, moves):
+        pool = Pool()
+        Date = pool.get('ir.date')
+
+        today = Date.today()
+        for move in moves:
+            move.effective_date = today
+            if (move.from_location.type in ('supplier', 'production')
+                    and move.to_location.type == 'storage'
+                    and move.product.cost_price_method == 'average'):
+                move._update_product_cost_price('out')
+            elif (move.to_location.type == 'supplier'
+                    and move.from_location.type == 'storage'
+                    and move.product.cost_price_method == 'average'):
+                move._update_product_cost_price('in')
+            move.effective_date = None
+            move.save()
+
+    @classmethod
+    def create(cls, vlist):
+        pool = Pool()
+        Product = pool.get('product.product')
+        Uom = pool.get('product.uom')
+
+        vlist = [x.copy() for x in vlist]
+        for vals in vlist:
+            assert vals.get('state', 'draft') == 'draft'
+
+            product = Product(vals['product'])
+            uom = Uom(vals['uom'])
+            internal_quantity = cls._get_internal_quantity(vals['quantity'],
+                uom, product)
+            vals['internal_quantity'] = internal_quantity
+        return super(Move, cls).create(vlist)
 
     @classmethod
     def write(cls, moves, vals):
-        Date = Pool().get('ir.date')
-
-        today = Date.today()
-        effective_date = vals.get('effective_date') or today
-
-        if 'state' in vals:
-            for move in moves:
-                if vals['state'] == 'cancel':
-                    vals['effective_date'] = None
-                    if (move.from_location.type in ('supplier', 'production')
-                            and move.to_location.type == 'storage'
-                            and move.state != 'cancel'
-                            and move.product.cost_price_method == 'average'):
-                        cls._update_product_cost_price(move.product.id,
-                                -move.quantity, move.uom, move.unit_price,
-                                move.currency, move.company, today)
-                    if (move.to_location.type == 'supplier'
-                            and move.from_location.type == 'storage'
-                            and move.state != 'cancel'
-                            and move.product.cost_price_method == 'average'):
-                        cls._update_product_cost_price(move.product.id,
-                                move.quantity, move.uom, move.unit_price,
-                                move.currency, move.company, today)
-
-                elif vals['state'] == 'draft':
-                    if move.state == 'done':
-                        cls.raise_user_error('set_state_draft')
-                elif vals['state'] == 'assigned':
-                    if move.state in ('cancel', 'done'):
-                        cls.raise_user_error('set_state_assigned')
-                    vals['effective_date'] = effective_date
-                elif vals['state'] == 'done':
-                    if move.state in ('cancel'):
-                        cls.raise_user_error('set_state_done')
-                    vals['effective_date'] = effective_date
-
-                    if (move.from_location.type in ('supplier', 'production')
-                            and move.to_location.type == 'storage'
-                            and move.state != 'done'
-                            and move.product.cost_price_method == 'average'):
-                        cls._update_product_cost_price(move.product.id,
-                                move.quantity, move.uom, move.unit_price,
-                                move.currency, move.company, effective_date)
-                    if (move.to_location.type == 'supplier'
-                            and move.from_location.type == 'storage'
-                            and move.state != 'done'
-                            and move.product.cost_price_method == 'average'):
-                        cls._update_product_cost_price(move.product.id,
-                                -move.quantity, move.uom, move.unit_price,
-                                move.currency, move.company, effective_date)
-
         if reduce(lambda x, y: x or y in vals, ('product', 'uom', 'quantity',
                 'from_location', 'to_location', 'company', 'unit_price',
                 'currency'), False):
             for move in moves:
                 if move.state in ('assigned', 'done', 'cancel'):
-                    cls.raise_user_error('modify_assigned_done_cancel')
+                    cls.raise_user_error('modify_assigned_done_cancel',
+                        (move.rec_name,))
         if reduce(lambda x, y: x or y in vals,
-                ('planned_date', 'effective_date'), False):
+                ('planned_date', 'effective_date', 'state'), False):
             for move in moves:
                 if move.state in ('done', 'cancel'):
-                    cls.raise_user_error('modify_assigned_done_cancel')
+                    cls.raise_user_error('modify_assigned_done_cancel',
+                        (move.rec_name,))
 
         super(Move, cls).write(moves, vals)
 
-        if vals.get('state', '') == 'done':
-            for move in moves:
-                if not move.cost_price:
-                    cls.write([move], {
-                            'cost_price': move.product.cost_price,
-                            })
-
         for move in moves:
             internal_quantity = cls._get_internal_quantity(move.quantity,
                     move.uom, move.product)
             if internal_quantity != move.internal_quantity:
-                # Use super to avoid infinite loop
-                super(Move, cls).write([move], {
+                cls.write([move], {
                         'internal_quantity': internal_quantity,
                         })
 
     @classmethod
     def delete(cls, moves):
         for move in moves:
-            if move.state not in  ('draft', 'cancel'):
-                cls.raise_user_error('del_draft_cancel')
+            if move.state not in ('draft', 'cancel'):
+                cls.raise_user_error('del_draft_cancel', (move.rec_name,))
         super(Move, cls).delete(moves)
 
     def pick_product(self, location_quantities):
@@ -595,13 +629,13 @@ class Move(ModelSQL, ModelView):
                 values = {
                     'from_location': from_location.id,
                     'quantity': qty,
-                    'state': 'assigned',
                     }
                 if first:
                     cls.write([move], values)
+                    cls.assign([move])
                     first = False
                 else:
-                    cls.copy([move], default=values)
+                    cls.assign(cls.copy([move], default=values))
 
                 qty_default_uom = Uom.compute_qty(move.uom, qty,
                         move.product.default_uom, round=False)
