@@ -19,8 +19,8 @@ class Period(ModelSQL, ModelView):
     company = fields.Many2One('company.company', 'Company', required=True,
         domain=[
             ('id', If(In('company', Eval('context', {})), '=', '!='),
-                    Get(Eval('context', {}), 'company', 0)),
-        ])
+                Get(Eval('context', {}), 'company', 0)),
+            ])
     caches = fields.One2Many('stock.period.cache', 'period', 'Caches')
     state = fields.Selection([
         ('draft', 'Draft'),
@@ -31,11 +31,11 @@ class Period(ModelSQL, ModelView):
     def __setup__(cls):
         super(Period, cls).__setup__()
         cls._error_messages.update({
-            'close_period_future_today': ('You can not close a period '
-                'in the future or today!'),
-            'close_period_assigned_move': ('You can not close a period when '
-                'there is still assigned moves!'),
-        })
+                'close_period_future_today': ('You can not close a period '
+                    'in the future or today.'),
+                'close_period_assigned_move': ('You can not close a period when '
+                    'there still are assigned moves.'),
+                })
         cls._buttons.update({
                 'draft': {
                     'invisible': Eval('state') == 'draft',
@@ -45,6 +45,10 @@ class Period(ModelSQL, ModelView):
                     },
                 })
 
+    @staticmethod
+    def default_state():
+        return 'draft'
+
     @classmethod
     @ModelView.button
     def draft(cls, periods):
@@ -88,6 +92,7 @@ class Period(ModelSQL, ModelView):
                         ]]):
             cls.raise_user_error('close_period_assigned_move')
 
+        to_create = []
         for period in periods:
             with Transaction().set_context(
                     stock_date_end=period.date,
@@ -98,12 +103,14 @@ class Period(ModelSQL, ModelView):
                     ):
                 pbl = Product.products_by_location([l.id for l in locations])
             for (location_id, product_id), quantity in pbl.iteritems():
-                Cache.create({
+                to_create.append({
                         'period': period.id,
                         'location': location_id,
                         'product': product_id,
                         'internal_quantity': quantity,
                         })
+        if to_create:
+            Cache.create(to_create)
         cls.write(periods, {
                 'state': 'closed',
                 })
